/*
  Copyright (c) 2015, Matthias Schiffer <mschiffer@universe-factory.net>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#pragma once

#include <string>
#include <tuple>


namespace solar {

enum symbol_type_t {
	SYMBOL_TYPE_NONTERM,
	SYMBOL_TYPE_TERM,
	SYMBOL_TYPE_CHAR,
};

struct symbol_t : public std::tuple<symbol_type_t, std::string>  {
	symbol_t(symbol_type_t type, const char *value) : std::tuple<symbol_type_t, std::string>(type, value) {}

	symbol_type_t get_type() const {
		return std::get<0>(*this);
	}

	const std::string & get_value() const {
		return std::get<1>(*this);
	}

	static symbol_t make_nonterm(const char *value) {
		return symbol_t(SYMBOL_TYPE_NONTERM, value);
	}

	static symbol_t make_term(const char *value) {
		return symbol_t(SYMBOL_TYPE_TERM, value);
	}

	static symbol_t make_char(unsigned char value) {
		char v[2] = {char(value), 0};
		return symbol_t(SYMBOL_TYPE_CHAR, v);
	}
};

}
